package rte;

public class DynamicRuntime {
  public static int objCnt;
  
  private final static int MEMBLOCK = 1024*1024; //1 MB per block
  private final static int BLOCKMAX = 64*1024;   //allocate objects up to 64 KB in memblock
  private static int nextFreeAddr, memFree;
  
  public static Object newInstance(int scalarSize, int relocEntries, SClassDesc type) {
    int addr, rs, size;
    Object me;
    
    //get informations
    rs=relocEntries*MAGIC.ptrSize;
    scalarSize=(scalarSize+MAGIC.ptrSize-1)&~(MAGIC.ptrSize-1);
    size=rs+scalarSize;
    if (size>BLOCKMAX) addr=allocMem(size); //allocate special block
    else { //allocate inside the pool
      if (size>memFree) {
        nextFreeAddr=allocMem(MEMBLOCK);
        memFree=MEMBLOCK;
      }
      addr=nextFreeAddr;
      memFree-=size;
      nextFreeAddr+=size;
    }
    //prepare object
    setMem32(addr, size>>2, 0); //clear memory
    me=MAGIC.cast2Obj(addr+rs); //place object
    me._r_type=type; //set type
    objCnt++;
    return me;
  }
  
  private static int allocMem(int size) {
    int result;
    
    result=brk(0);
    if (result<=0) {
      kernel.Viewer.print("Error allocating memory");
      exit(-1);
    }
    size=(size+0xFFF)&0xFFFFF000;
    brk(result+size);
    return result;
  }
  
  private static void setMem32(int addr, int cnt, int val) {
    MAGIC.inline(0x57);             //push edi
    MAGIC.inline(0xFC);             //cld
    MAGIC.inline(0x8B, 0x7D, 0x10); //mov edi,[ebp+16]
    MAGIC.inline(0x8B, 0x45, 0x08); //mov eax,[ebp+8]
    MAGIC.inline(0x8B, 0x4D, 0x0C); //mov ecx,[ebp+12]
    MAGIC.inline(0xF3, 0xAB);       //rep stosd
    MAGIC.inline(0x5F);             //pop edi
  }
  
  public static SArray newArray(int length, int arrDim, int entrySize, int stdType, Object unitType) {
    int scS, rlE;
    SArray me;
    
    scS=MAGIC.getInstScalarSize("SArray");
    rlE=MAGIC.getInstRelocEntries("SArray");
    if (arrDim>1 || entrySize<0) rlE+=length; //objects => rlE will become bigger
    else scS+=length*entrySize; //entrySize>0 => scS will become bigger
    me=(SArray)newInstance(scS, rlE, MAGIC.clssDesc("SArray"));
    me.length=length;
    me._r_dim=arrDim;
    me._r_stdType=stdType;
    me._r_unitType=unitType;
    return me;
  }
  
  public static void newMultArray(SArray[] parent, int curLevel, int destLevel,
      int length, int arrDim, int entrySize, int stdType, Object unitType) {
    int i;
    
    if (curLevel+1<destLevel) { //step down one level
      curLevel++;
      for (i=0; i<parent.length; i++) {
        newMultArray((SArray[])((Object)parent[i]), curLevel, destLevel,
            length, arrDim, entrySize, stdType, unitType);
      }
    }
    else { //create the new entries
      destLevel=arrDim-curLevel;
      for (i=0; i<parent.length; i++) {
        parent[i]=newArray(length, destLevel, entrySize, stdType, unitType);
      }
    }
  }
  
  public static boolean isInstance(Object o, SClassDesc dest, boolean asCast) {
    SClassDesc check;
    
    if (o==null) {
      if (asCast) return true; //null matches all
      return false; //null is not an instance
    }
    check=o._r_type;
    while (check!=null) {
      if (check==dest) return true;
      check=check.parent;
    }
    if (asCast) rtError();
    return false;
  }
  
  public static SIntfMap isImplementation(Object o, SIntfDesc dest, boolean asCast) {
    rtError();
    return null;
  }
  
  public static boolean isArray(SArray o, int stdType, Object unitType, int arrDim, boolean asCast) {
    SClassDesc clss;
    
    //in fact o is of type "Object", _r_type has to be checked below - but this check is faster than "instanceof" and conversion
    if (o==null) {
      if (asCast) return true; //null matches all
      return false; //null is not an instance
    }
    if (o._r_type!=MAGIC.clssDesc("SArray")) { //will never match independently of arrDim
      if (asCast) rtError();
      return false;
    }
    if (unitType==MAGIC.clssDesc("SArray")) { //special test for arrays
      if (o._r_unitType==MAGIC.clssDesc("SArray")) arrDim--; //an array of SArrays, make next test to ">=" instead of ">"
      if (o._r_dim>arrDim) return true; //at least one level has to be left to have an object of type SArray
      if (asCast) rtError();
      return false;
    }
    //no specials, check arrDim and check for standard type
    if (o._r_stdType!=stdType || o._r_dim<arrDim) { //check standard types and array dimension
      if (asCast) rtError();
      return false;
    }
    if (stdType!=0) return true; //array of standard-type
    //array of objects, make deep-check for class type
    clss=(SClassDesc)o._r_unitType;
    while (clss!=null) {
      if (clss==unitType) return true;
      clss=clss.parent;
    }
    if (asCast) rtError();
    return false;
  }
  
  public static void checkArrayStore(SArray dest, SArray newEntry) {
    //in fact newEntry is of type "Object", SArray elements are not valid always - but this is faster for _r_dim>1 which needs SArray parm
    if (dest._r_dim>1) isArray(newEntry, dest._r_stdType, dest._r_unitType, dest._r_dim-1, true); //check array
    else if (dest._r_unitType==null) rtError();
    else {
      if (dest._r_unitType._r_type==MAGIC.clssDesc("SClassDesc")) {
        isInstance(newEntry, (SClassDesc)dest._r_unitType, true); //check instance
      }
      else {
        isImplementation(newEntry, (SIntfDesc)dest._r_unitType, true); //check implementation
      }
    }
  }
  
  public static void rtError() {
    kernel.Viewer.println("Runtime error");
    exit(-1);
  }

  public static void exit(int status) {
    MAGIC.inline(0xB8, 0x01, 0x00, 0x00, 0x00); //mov eax,1 (terminate process)
    MAGIC.inline(0x8B, 0x5D, 0x08);             //mov ebx,[ebp+8] (exit code)
    MAGIC.inline(0xCD, 0x80);                   //call kernel
  }
  
  private static int brk(int newEnd) {
    int tmp=0;
    MAGIC.inline(0xB8, 0x2D, 0x00, 0x00, 0x00); //mov eax,45 (brk)
    MAGIC.inline(0x8B, 0x5D, 0x08);             //mov ebx,[ebp+8] (end_data_segment)
    MAGIC.inline(0xCD, 0x80);                   //call kernel
    MAGIC.inline(0x89, 0x45, 0xFC);             //mov [ebp-4],eax
    return tmp;
  }
}

package rte;

public class SArray {
  public int length, _r_dim, _r_stdType;
  public Object _r_unitType;
}

package rte;

public class SClassDesc {
  public SClassDesc parent;
  public SIntfMap implementations;
}

package rte;

public class SIntfDesc {
}

package rte;

public class SIntfMap {
  public SIntfDesc owner;
  public SIntfMap next;
}
package rte;

public class SMthdBlock {
}

package java.lang;

public class IGNORECLASS {

}

package java.lang;

import rte.SClassDesc;

public class Object {
  public SClassDesc _r_type; //fixed by compiler: reloc-position 0
}
package java.lang;

public class String {
  public char[] value;
  public int count;

  public int length() {
    return count;
  }
}

package java.lang;

@SJC.IgnoreUnit
public class STRUCT {

}

package java.lang;

public class Math {
  public final static double PI = 3.141592653589793238;
  
  public static double tan(double nr) {
    MAGIC.inline(0xDD, 0x45); MAGIC.inlineOffset(1, nr); //fld qword [ebp+8]
    MAGIC.inline(0xD9, 0xF2);       //fptan (puts additional "1.0" onto the stack)
    MAGIC.inline(0xDD, 0x5D); MAGIC.inlineOffset(1, nr); //fstp qword [ebp+8] (remove 1.0)
    MAGIC.inline(0xDD, 0x5D); MAGIC.inlineOffset(1, nr); //fstp qword [ebp+8]
    return nr;
  }
  
  public static double sqrt(double nr) {
    MAGIC.inline(0xDD, 0x45); MAGIC.inlineOffset(1, nr); //fld qword [ebp+8]
    MAGIC.inline(0xD9, 0xFA);       //fsqrt
    MAGIC.inline(0xDD, 0x5D); MAGIC.inlineOffset(1, nr); //fstp qword [ebp+8]
    return nr;
  }
  
  public static double exp(double nr) {
    int dummy=0; //holds FPU state
    MAGIC.ignore(dummy);
    MAGIC.inline(0xD9, 0x7D); MAGIC.inlineOffset(1, dummy, 2); //fnstcw word [ebp-2]
    MAGIC.inline(0xD9, 0x7D); MAGIC.inlineOffset(1, dummy, 0); //fnstcw word [ebp-4]
    MAGIC.inline(0x80, 0x4D); MAGIC.inlineOffset(1, dummy, 1); MAGIC.inline(0x0B); //or byte [ebp-3],0x0B
    MAGIC.inline(0xD9, 0x6D); MAGIC.inlineOffset(1, dummy); //fldcw word [ebp-4]
    MAGIC.inline(0xDD, 0x45); MAGIC.inlineOffset(1, nr); //fld qword [ebp+8]
    MAGIC.inline(0xD9, 0xEA);             //fldl2e
    MAGIC.inline(0xDE, 0xC9);             //fmulp st(1),st
    MAGIC.inline(0xD9, 0xC0);             //fld st(0)
    MAGIC.inline(0xD9, 0xFC);             //frndint
    MAGIC.inline(0xD9, 0xC9);             //fxch st(1)
    MAGIC.inline(0xD8, 0xE1);             //fsub st,st(1)
    MAGIC.inline(0xD9, 0xF0);             //f2xm1
    MAGIC.inline(0xD9, 0xE8);             //fld1
    MAGIC.inline(0xDE, 0xC1);             //faddp st(1),st
    MAGIC.inline(0xD9, 0xC9);             //fxch st(1)
    MAGIC.inline(0xD9, 0xE8);             //fld1
    MAGIC.inline(0xD9, 0xFD);             //fscale
    MAGIC.inline(0xDD, 0xD9);             //fstp st(1)
    MAGIC.inline(0xDE, 0xC9);             //fmulp st(1),st
    MAGIC.inline(0xD9, 0x6D); MAGIC.inlineOffset(1, dummy, 2); //fldcw word [ebp-2]
    MAGIC.inline(0xDD, 0x5D); MAGIC.inlineOffset(1, nr); //fstp qword [ebp+8]
    return nr;
  }
  
  public static double ln(double nr) {
    MAGIC.inline(0xD9, 0xE8);       //fld1
    MAGIC.inline(0xDD, 0x45); MAGIC.inlineOffset(1, nr); //fld qword [ebp+8]
    MAGIC.inline(0xD9, 0xF1);       //fyl2x
    MAGIC.inline(0xD9, 0xEA);       //fldl2e
    MAGIC.inline(0xDE, 0xF9);       //fdivp
    MAGIC.inline(0xDD, 0x5D); MAGIC.inlineOffset(1, nr); //fstp qword [ebp+8]
    return nr;
  }
  
  public static double pow(double nr, double exp) {
    return exp(exp*ln(nr));
  }
  
  public static double abs(double nr) {
    MAGIC.inline(0xDD, 0x45); MAGIC.inlineOffset(1, nr); //fld qword [ebp+8]
    MAGIC.inline(0xD9, 0xE1);       //fabs
    MAGIC.inline(0xDD, 0x5D); MAGIC.inlineOffset(1, nr); //fstp qword [ebp+8]
    return nr;
  }
}
package kernel;

public class Viewer {
  public static void printChar(int c) {
    MAGIC.inline(0xB8, 0x04, 0x00, 0x00, 0x00); //mov eax,4 (print string)
    MAGIC.inline(0xBB, 0x01, 0x00, 0x00, 0x00); //mov ebx,1 (handle for std-out)
    MAGIC.inline(0x8D, 0x4D, 0x08);             //lea ecx,[ebp+8] (address of string)
    MAGIC.inline(0x89, 0xDA);                   //mov edx,ebx (length of string: 1)
    MAGIC.inline(0xCD, 0x80);                   //call kernel
    //sync for debugging
    //MAGIC.inline(0xB8, 0x94, 0x00, 0x00, 0x00); //mov eax,148 (sync file)
    //MAGIC.inline(0xBB, 0x01, 0x00, 0x00, 0x00); //mov ebx,1 (handle for std-out)
    //MAGIC.inline(0xCD, 0x80);                   //call kernel
  }
  
  public static void print(int i) {
    if (i<0) {
      printChar(45);
      i=-i;
    }
    if (i==0) printChar(48);
    else {
      if (i>=10) print(i/10);
      printChar(48+i%10);
    }
  }
  
  public static void printHex(int i) {
    int v, p;
    
    for (p=0; p<8; p++) {
      v=(i>>>((7-p)<<2))&0xF;
      if (v<10) printChar(48+v); //'0'..'9'
      else printChar(55+v); //'A'..'Z'
    }
  }
  
  public static void printHexByte(int b) {
    int v, p;
    
    for (p=0; p<2; p++) {
      v=(b>>>((1-p)<<2))&0xF;
      if (v<10) printChar(48+v); //'0'..'9'
      else printChar(55+v); //'A'..'Z'
    }
  }
  
  public static void printHexLong(long l) {
    printHex((int)(l>>>32));
    printHex((int)l);
  }
  
  public static void print(String what) {
    int i;
    
    for (i=0; i<what.count; i++) printChar((int)what.value[i]);
  }

  public static void printBZ(byte[] bzStr) {
    int i;

    for (i=0; i<bzStr.length && bzStr[i]!=0; i++)
      printChar((int)bzStr[i]);
  }
  
  public static void println() {
    printChar(10);
  }

  public static void println(String what) {
    print(what);
    printChar(10);
  }
}

package kernel;

public class Kernel {
  public static void main() {
    Window.run();
  }
}

package kernel;

import raytracer.RayTrace;
import raytracer.Scene;
import raytracer.CalcTAinfo;

public class Window {
  
  public final static int XRES=800, YRES=600;
  
  public final static int RTLD_NOW = 0x00002;
  public final static int RTLD_GLOBAL = 0x00100;

  public final static int ZPixmap = 2;

  public final static int ButtonPressMask = 0x00000004;
  public final static int ButtonPress = 4;
  public final static int ExposureMask = 0x00008000;
  public final static int Expose = 12;

  public final static int XA_STRING = 31;

  public static int XOpenDisplay;
  public static int XDisplayWidth, XDisplayHeight, XDisplayDepth;
  public static int XDefaultScreen, XDefaultRootWindow;
  public static int XCreateSimpleWindow, XMapWindow, XSetWMName, XCloseDisplay;
  public static int XDefaultGC;
  public static int XFlush;
  public static int XSelectInput, XNextEvent, XCheckWindowEvent;
  public static int XSetForeground, XDrawLine;
  public static int XDefaultVisual, XCreateImage, XPutImage;

  /**
   * Method initializing the functions needed for this program
   */
  private static void initFunctions() {
    if (MAGIC.rMem32(MAGIC.imageBase-12)==0 || MAGIC.rMem32(MAGIC.imageBase-8)==0) {
      Viewer.println("no library function loaded");
      rte.DynamicRuntime.exit(-1);
    }
    int lib=dlopen(MAGIC.toByteArray("libX11.so.6", true), RTLD_NOW|RTLD_GLOBAL);
    if ((XOpenDisplay=dlsym(lib, MAGIC.toByteArray("XOpenDisplay", true)))==0
        || (XDisplayWidth=dlsym(lib, MAGIC.toByteArray("XDisplayWidth", true)))==0
        || (XDisplayHeight=dlsym(lib, MAGIC.toByteArray("XDisplayHeight", true)))==0
        || (XDisplayDepth=dlsym(lib, MAGIC.toByteArray("XDefaultDepth", true)))==0
        || (XDefaultScreen=dlsym(lib, MAGIC.toByteArray("XDefaultScreen", true)))==0
        || (XDefaultRootWindow=dlsym(lib, MAGIC.toByteArray("XDefaultRootWindow", true)))==0
        || (XCreateSimpleWindow=dlsym(lib, MAGIC.toByteArray("XCreateSimpleWindow", true)))==0
        || (XMapWindow=dlsym(lib, MAGIC.toByteArray("XMapWindow", true)))==0
        || (XSetWMName=dlsym(lib, MAGIC.toByteArray("XSetWMName", true)))==0
        || (XCloseDisplay=dlsym(lib, MAGIC.toByteArray("XCloseDisplay", true)))==0
        || (XDefaultGC=dlsym(lib, MAGIC.toByteArray("XDefaultGC", true)))==0
        || (XFlush=dlsym(lib, MAGIC.toByteArray("XFlush", true)))==0
        || (XSelectInput=dlsym(lib, MAGIC.toByteArray("XSelectInput", true)))==0
        || (XNextEvent=dlsym(lib, MAGIC.toByteArray("XNextEvent", true)))==0
        || (XCheckWindowEvent=dlsym(lib, MAGIC.toByteArray("XCheckWindowEvent", true)))==0
        || (XSetForeground=dlsym(lib, MAGIC.toByteArray("XSetForeground", true)))==0
        || (XDrawLine=dlsym(lib, MAGIC.toByteArray("XDrawLine", true)))==0
        || (XDefaultVisual=dlsym(lib, MAGIC.toByteArray("XDefaultVisual", true)))==0
        || (XCreateImage=dlsym(lib, MAGIC.toByteArray("XCreateImage", true)))==0
        || (XPutImage=dlsym(lib, MAGIC.toByteArray("XPutImage", true)))==0) {
      Viewer.println("function resolving failed");
      rte.DynamicRuntime.exit(-1);
    }
  }

  public static int dlopen(byte[] libNameZ, int flags) {
    int libAddr=MAGIC.addr(libNameZ[0]);

    MAGIC.inline(0xFF, 0x75); MAGIC.inlineOffset(1, flags);
    MAGIC.inline(0xFF, 0x75); MAGIC.inlineOffset(1, libAddr);
    MAGIC.inline(0xFF, 0x15); MAGIC.inline32(MAGIC.imageBase-12);
    MAGIC.inline(0x89, 0x45); MAGIC.inlineOffset(1, libAddr);
    MAGIC.inline(0x5B, 0x5B);
    return libAddr;
  }

  public static int dlsym(int libHandle, byte[] funcNameZ) {
    int res=dlsymReal(libHandle, funcNameZ);
    if (res!=0) return res;
    Viewer.printBZ(funcNameZ);
    Viewer.println(" missing");
    return 0;
  }

  public static int dlsymReal(int libHandle, byte[] funcNameZ) {
    int funcAddr=MAGIC.addr(funcNameZ[0]);

    MAGIC.inline(0xFF, 0x75); MAGIC.inlineOffset(1, funcAddr);
    MAGIC.inline(0xFF, 0x75); MAGIC.inlineOffset(1, libHandle);
    MAGIC.inline(0xFF, 0x15); MAGIC.inline32(MAGIC.imageBase-8);
    MAGIC.inline(0x89, 0x45); MAGIC.inlineOffset(1, funcAddr);
    MAGIC.inline(0x5B, 0x5B);
    return funcAddr;
  }

  public native static int XOpenDisplay(int no);
  public native static int XDisplayWidth(int display, int screen);
  public native static int XDisplayHeight(int display, int screen);
  public native static int XDisplayDepth(int display, int screen);
  public native static int XDefaultScreen(int display);
  public native static int XDefaultRootWindow(int display);
  public native static int XCreateSimpleWindow(int display, int root, int x, int y,
      int width, int height, int unknown, int color1, int color2);
  public native static void XMapWindow(int display, int window);
  public native static void XSetWMName(int display, int window, XTextProperty text_prop);
  public native static void XCloseDisplay(int display);
  public native static int XDefaultGC(int display, int unknown);
  public native static void XFlush(int display);
  public native static void XSelectInput(int display, int window, int mask);
  public native static void XNextEvent(int display, XEvent event);
  public native static int XCheckWindowEvent(int display, int window, int mask, XEvent event);
  public native static void XSetForeground(int display, int gc, int color);
  public native static void XDrawLine(int display, int window, int gc,
      int x1, int y1, int x2, int y2);
  public native static int XDefaultVisual(int display, int screen);
  public native static int XCreateImage(int display, int visual, int depth, int format,
      int offset, int data, int width, int height, int pad, int bytes_per_line);
  public native static void XPutImage(int display, int window, int gc, int image,
      int srcX, int srcY, int dstX, int dstY, int width, int height);

  /**
   * Method starting the raytracer
   *
   */
  public static void run() {
    //-- fields for image processing
    RayTrace rt;
    CalcTAinfo cTAi;
    
    //---
    // init all needed functions
    initFunctions();
    
    int display=XOpenDisplay(0);
    if (display==0) {
      Viewer.println("could not open default display");
      return;
    }
    int colorDepth=XDisplayDepth(display, XDefaultScreen(display));
    if (colorDepth!=24 && colorDepth!=32) Viewer.println("current demo requires 24 or 32 bit color depth on screen");

    int mainWindow=XCreateSimpleWindow(display, XDefaultRootWindow(display), 0, 0, XRES, YRES, 0, 0, 0);
    if (mainWindow==0) {
      Viewer.println("could not create main window");
      return;
    }
    int mainGC=XDefaultGC(display, 0);
    XMapWindow(display, mainWindow);
    XTextProperty windowName;
    windowName=(XTextProperty)MAGIC.cast2Struct(MAGIC.addr((new int[4])[0]));
    windowName.stringAddr = MAGIC.addr(MAGIC.toByteArray("LinRay", true)[0]);
    windowName.encoding   = XA_STRING;
    windowName.format     = 8;
    windowName.nitems     = 6;
    XSetWMName(display, mainWindow, windowName);
    XFlush(display);

    int[][] image=new int[YRES][XRES];
    rt=new RayTrace(new Scene());
    rt.init(XRES, YRES);
    cTAi=new CalcTAinfo();
    
    XSelectInput(display, mainWindow, ButtonPressMask | ExposureMask);

    int ximage=0;
    int[] ximageData=new int[YRES*XRES];
    int bitmapBytes=MAGIC.addr(ximageData[0]);
    XEvent event;
    XButtonEvent buttonEvent;
    event=(XEvent)MAGIC.cast2Struct(MAGIC.addr((new byte[96])[0]));
    buttonEvent=(XButtonEvent)MAGIC.cast2Struct(MAGIC.cast2Ref(event));
    if ((ximage=XCreateImage(display, XDefaultVisual(display, XDefaultScreen(display)), colorDepth, ZPixmap, 0,
        bitmapBytes, XRES, YRES, 8, XRES*4))==0) {
      Viewer.println("could not create image");
      return;
    }
    
    int line[];
    int linesDone=0;
    boolean goOn=true;
    boolean wait=false; //do not wait for messages until calculation is done
    while (goOn) { //keep running until procWind tells us to leave
      if (linesDone<YRES) { //something left to calculate
        rt.renderLine(line=image[cTAi.y=linesDone], cTAi);
        memCpy32(MAGIC.addr(line[0]), linesDone*(XRES*4)+bitmapBytes, XRES);
        XPutImage(display, mainWindow, mainGC, ximage, 0, linesDone, 0, linesDone++, XRES, 1);
        XFlush(display);
      }
      else wait=true; //nothing more to calculate, wait for messages now
      if (wait || XCheckWindowEvent(display, mainWindow, -1, event)!=0) { //no wait, check if event existing
        if (wait) XNextEvent(display, event); //get event if we have to wait for it
        switch (event.type) {
          case Expose:
            XPutImage(display, mainWindow, mainGC, ximage, 0, 0, 0, 0, XRES, linesDone);
            XFlush(display);
            break;
          case ButtonPress:
            if (buttonEvent.window==mainWindow) {
              XCloseDisplay(display);
              goOn=false;
            }
            break;
        }
      }
    }
  }
  
  private final static void memCpy32(int src, int dst, int dwordCnt) {
    MAGIC.inline(0x56);             //push esi
    MAGIC.inline(0x57);             //push edi
    MAGIC.inline(0xFC);             //cld
    MAGIC.inline(0x8B, 0x75); MAGIC.inlineOffset(1, src); //mov esi,[ebp+16]
    MAGIC.inline(0x8B, 0x7D); MAGIC.inlineOffset(1, dst); //mov edi,[ebp+12]
    MAGIC.inline(0x8B, 0x4D); MAGIC.inlineOffset(1, dwordCnt); //mov ecx,[ebp+8]
    MAGIC.inline(0xF3, 0xA5);       //rep movsd
    MAGIC.inline(0x5F);             //pop edi
    MAGIC.inline(0x5E);             //pop esi
  }
}

package kernel;

public class XTextProperty extends STRUCT {
    int stringAddr;	/* same as Property routines */
    int encoding;	/* prop type */
    int format;		/* prop data format: 8, 16, or 32 */
    int nitems;		/* number of data items in value */
}

package kernel;

public class XEvent extends STRUCT {
  int type;
}

package kernel;

public class XButtonEvent extends STRUCT {
  int type; /* has to be ButtonPress */
  int serial; /* # of last request processed by server */
  int send_event;  /* true if this came from a SendEvent request */
  int display;  /* Display the event was read from */
  int window;          /* "event" window it is reported relative to */
  int root;          /* root window that the event occurred on */
  int subwindow;  /* child window */
  int time;    /* milliseconds */
  int x, y;    /* pointer x, y coordinates in event window */
  int x_root, y_root;  /* coordinates relative to root */
  int state;  /* key or button mask */
  int button;  /* detail */
  int same_screen;  /* same screen flag */
}

package raytracer;

public class CalcTAinfo {
  public int x, y; // pixel to calc in CalcDef.render(.)
  public Object var1, var2, var3, var4, var5; // temporary variables for potentially
                                              // use in CalcDef.render(.)
}
package raytracer;

public class Cylinder extends Renderable {
  public Vector3f origin;
  public Vector3f direction; // normed vector
  public float radiusSq;
  public Surface surface;

  public Cylinder(Surface is, Vector3f io, float ir, Vector3f id, boolean idCopy) {
    surface = is;
    origin = io;
    radiusSq = ir * ir;
    if (idCopy) {
      direction = new Vector3f();
      direction.normalize(id);
    }
    else {
      direction = id;
      direction.normalize();
    }
  }

  public float intersect(Ray ray) {
    float a, b, c, sc;
    float t, det, sqrtDet, tMin, tMax;
    float ax, ay, az, bx, by, bz; // this should be alpha, beta - but we don't
                                  // want "new"

    ax = ray.origin.x - origin.x;
    ay = ray.origin.y - origin.y;
    az = ray.origin.z - origin.z;
    sc = ax * direction.x + ay * direction.y + az * direction.z;
    ax -= direction.x * sc;
    ay -= direction.y * sc;
    az -= direction.z * sc;

    bx = ray.direction.x;
    by = ray.direction.y;
    bz = ray.direction.z;
    sc = bx * direction.x + by * direction.y + bz * direction.z;
    bx -= direction.x * sc;
    by -= direction.y * sc;
    bz -= direction.z * sc;

    a = Vector3f.lenSq(bx, by, bz);
    b = 2.0f * (ax * bx + ay * by + az * bz);
    c = Vector3f.lenSq(ax, ay, az) - radiusSq;

    if (a == 0.0f) {
      if (b != 0.0f) {
        t = (-c) / b;
        if (t > 0.0f)
          return onPoint(t);
      }
      return -1.0f;
    }

    det = b * b - 4.0f * a * c;
    if (det <= 0.0f)
      return -1.0f;

    if ((sqrtDet = (float) Math.sqrt((double) det)) < 0.0f) {
      tMin = (-b + sqrtDet) / (2.0f * a);
      tMax = (-b - sqrtDet) / (2.0f * a);
    }
    else {
      tMin = (-b - sqrtDet) / (2.0f * a);
      tMax = (-b + sqrtDet) / (2.0f * a);
    }
    if (tMax < 0.0f)
      return -1.0f;
    if (tMin > 0.0f)
      return onPoint(tMin);
    return onPoint(tMax);
  }

  public long Shade(Ray ray, Scene scene, Vector3f t1, Vector3f t2, Vector3f t3) {
    // An object shader doesn't really do too much other than
    // supply a few critical pieces of geometric information
    // for a surface shader. It must must compute
    // 1. the point of intersection (p)
    // 2. a unit-length surface normal (n)
    // 3. a unit-length vector towards the ray's origin (v)

    float t = onPoint(ray.t);
    float px = ray.origin.x + t * ray.direction.x;
    float py = ray.origin.y + t * ray.direction.y;
    float pz = ray.origin.z + t * ray.direction.z;
    float sc;
    long col;

    t1.init(px, py, pz);
    t2.init(px - origin.x, py - origin.y, pz - origin.z);
    sc = t2.dot(direction);
    t2.addScaled(direction, -sc); // calculate the projection with our direction
    t2.normalize();
    t3.init(-ray.direction.x, -ray.direction.y, -ray.direction.z);

    // The illumination model is applied
    // by the surface's Shade() method
    col = surface.Shade(t1, t2, t3, scene, ray);

    return col;
  }
}
package raytracer;

/**
 * The Light class represents a light emitter in the scene. Lights can be of
 * multiple types (ambient, directional, point). Light has no methods except a
 * constructor; it is essentially a data structure.
 * 
 * There are loads of public variables so that lights and surfaces can get
 * along; these should probably be abstracted.
 */

public class Light {
  /** denotes ambient light */
  public static final int AMBIENT = 0;

  /** denotes directional light (not a spotlight) */
  public static final int DIRECTIONAL = 1;

  /** denotes a hypothetical but ever-present point light */
  public static final int POINT = 2;

  /** stores the type of light, as per enumerations above */
  public int lightType;

  /**
   * This stores the position of a point light, or, the direction of a
   * directional light.
   */
  public Vector3f lvec;

  /**
   * These hold the intensity of the light source in red, green, blue
   */
  public float ir, ig, ib;

  /**
   * Sole constructor; only operation is to normalize a directional light's
   * direction and clamp intensities.
   * 
   * @param int
   *          the type of light
   * @param Vector3f
   *          the position or direction
   * @param float
   *          red light color, from 0 to 1
   * @param float
   *          green light color, from 0 to 1
   * @param float
   *          blue light color, from 0 to 1
   */
  public Light(int type, Vector3f v, float r, float g, float b) {
    lightType = type;
    ir = r;
    ig = g;
    ib = b;

    // clamp intensity values
    if (ir < 0f)
      ir = 0f;
    if (ig < 0f)
      ig = 0f;
    if (ib < 0f)
      ib = 0f;
    if (ir > 1f)
      ir = 1f;
    if (ig > 1f)
      ig = 1f;
    if (ib > 1f)
      ib = 1f;

    if (type != AMBIENT) {
      lvec = v;
      if (type == DIRECTIONAL)
        lvec.normalize();
    }
  }
}
package raytracer;

public class ObjList {
  public Object elem;
  public ObjList next;

  public ObjList() {
    elem = null;
    next = null;
  }

  public ObjList(Object obj) {
    elem = obj;
    next = null;
  }

  public void addElement(Object obj) {
    ObjList myList;

    if (elem == null) {
      elem = obj;
      // next=null; //is already initialized
    }
    else {
      myList = this;
      while (myList.next != null)
        myList = myList.next;
      myList.next = new ObjList(obj);
    }
  }
}
package raytracer;

/**
 * The Ray class is a standard geometric ray; it contains an origin point and a
 * direction. It also has support for object intersection and tracing. It can be
 * treated as a parametric line, where 0 is the origin and positive t moves in
 * the direction of the ray.
 */

public class Ray {
  /**
   * MAX_T is the farthest distance the ray can travel.
   */
  public float MAX_T = 1e38f; // should be Float.MAX_VALUE

  /** origin point of the ray */
  public Vector3f origin;

  /** normalized direction of the ray */
  public Vector3f direction;

  /** parametric t value */
  public float t;

  /**
   * This is the sole constructor; it normalizes the direction vector.
   * 
   * @param Vector3f
   *          eyepoint
   * @param Vector3f
   *          direction
   */
  public Ray(Vector3f eye, Vector3f dir) {
    origin = new Vector3f();
    direction = new Vector3f();
    init(eye, dir);
  }

  public void init(Vector3f eye, Vector3f dir) {
    origin.init(eye.x, eye.y, eye.z);
    direction.normalize(dir);
  }

  /**
   * This function casts the ray from the origin to infinity, stopping when it
   * hits any object.
   * 
   * @param Vector
   *          of objects to test for intersection
   * @return boolean whether any object was hit
   */
  public boolean cast(ObjList objList) {
    // It's really only useful for shadow casting.

    float tval = 0.0f;
    Renderable objTemp;

    t = MAX_T;
    // object = null;
    while (objList != null) {
      objTemp = (Renderable) objList.elem;
      objList = objList.next;
      tval = objTemp.intersect(this);

      // if returns a t > 0, we have a hit
      if (tval > 0.0f)
        return true;
    }

    // if we get here, we never hit any object
    return false;
  }

  /**
   * This function traces the ray from the origin to infinity, checking every
   * object to determine which is closest.
   * 
   * @param Vector
   *          of objects to test for intersection
   * @return boolean whether any object was hit
   */
  public Renderable trace(ObjList objList) {
    Renderable objTemp, objRet;
    float tval = 0.0f;
    t = MAX_T;

    objRet = null;
    while (objList != null) {
      objTemp = (Renderable) objList.elem;
      objList = objList.next;
      tval = objTemp.intersect(this);

      // V1.1: Changed the condition so that it will
      // test every single object in the world, not just
      // the first.

      // if returns a t > 0, we have a hit
      if (tval > 0.0f && tval < t) {
        t = tval;
        objRet = objTemp;
      }
    }

    return objRet;
  }
}
package raytracer;

/**
 * The following application demonstrates a simple ray tracer.
 */
public class RayTrace {
  /** horizontal direction of screen in worldspace */
  public Vector3f Du = new Vector3f();

  /** vertical direction of screen in worldspace */
  public Vector3f Dv = new Vector3f();

  /** vector from eye to topleft corner of screen in worldspace */
  public Vector3f Vp = new Vector3f();

  /** the world description, with objects, lights, and camera info */
  public Scene scene;

  /** the screen size */
  public int width, height;
  
  /** bgr instead of rgb color model */
  public boolean bgr;

  /**
   * Initializes an instance of the raytracer with the window size and the world
   * description.
   * 
   * @param int
   *          image width
   * @param int
   *          image height
   * @param Scene
   *          world description
   */
  public RayTrace(Scene s) {
    scene = s;
  }

  public void init(int w, int h) {
    float fl;
    Vector3f look, crossp;

    width = w;
    height = h;
    // Compute viewing matrix that maps a
    // screen coordinate to a ray direction
    look = new Vector3f(scene.lookat.x - scene.eye.x, scene.lookat.y
        - scene.eye.y, scene.lookat.z - scene.eye.z);
    crossp = new Vector3f();
    crossp.cross(look, scene.up);
    Du.normalize(crossp);
    crossp.cross(look, Du);
    Dv.normalize(crossp);
    // fl = (width / (2.0f*FPU.tan((0.5f*scene.fov)*FPU.PI/180.0f)));
    fl = ((float) width)
        / (2.0f * (float) Math.tan((double) ((0.5f * scene.fov)
            * (float) Math.PI / 180.0f)));
    // fl=(0.5f*scene.fov)*FloatUtil.PI()/180.0f;
    // fl=((float)width) / (2.0f*FPU.sin(fl)/FPU.cos(fl));
    Vp.normalize(look);
    Vp.x = Vp.x * fl - 0.5f * ((float) width * Du.x + (float) height * Dv.x);
    Vp.y = Vp.y * fl - 0.5f * ((float) width * Du.y + (float) height * Dv.y);
    Vp.z = Vp.z * fl - 0.5f * ((float) width * Du.z + (float) height * Dv.z);
    
    //prepare scene
    // Compute viewing matrix that maps a
    // screen coordinate to a ray direction
    look = new Vector3f(scene.lookat.x - scene.eye.x, scene.lookat.y
        - scene.eye.y, scene.lookat.z - scene.eye.z);
    crossp = new Vector3f();
    crossp.cross(look, scene.up);
    Du.normalize(crossp);
    crossp.cross(look, Du);
    Dv.normalize(crossp);
    fl = (float) width
        / (2.0f * (float) Math.tan((double) ((0.5f * scene.fov)
            * (float) Math.PI / 180.0f)));
    Vp.normalize(look);
    Vp.x = Vp.x * fl - 0.5f * ((float) width * Du.x + (float) height * Dv.x);
    Vp.y = Vp.y * fl - 0.5f * ((float) width * Du.y + (float) height * Dv.y);
    Vp.z = Vp.z * fl - 0.5f * ((float) width * Du.z + (float) height * Dv.z);
  }
  
  /**
   * Render a line of pixel, by creating ray through the top-left corner of the
   * requested pixel, tracing the ray through the scene, and shading
   * accordingly.
   */
  public void renderLine(int[] dest, CalcTAinfo cTAi) {
    Vector3f dir;
    Ray ray;
    Renderable rayObject;
    Vector3f t1, t2, t3;
    int x;
    
    if (cTAi.var1 != null) dir = (Vector3f) cTAi.var1;
    else cTAi.var1 = dir = new Vector3f();
    if (cTAi.var2 != null) ray = (Ray) cTAi.var2;
    else cTAi.var2 = ray = new Ray(scene.eye, dir);
    if (cTAi.var3 != null) t1 = (Vector3f) cTAi.var3;
    else cTAi.var3 = t1 = new Vector3f();
    if (cTAi.var4 != null) t2 = (Vector3f) cTAi.var4;
    else cTAi.var4 = t2 = new Vector3f();
    if (cTAi.var5 != null) t3 = (Vector3f) cTAi.var5;
    else cTAi.var5 = t3 = new Vector3f();
    
    for (x=0; x<width; x++) {
      cTAi.x=x;
      dir.init((float) cTAi.x * Du.x + (float) cTAi.y * Dv.x + Vp.x,
          (float) cTAi.x * Du.y + (float) cTAi.y * Dv.y + Vp.y, (float) cTAi.x
              * Du.z + (float) cTAi.y * Dv.z + Vp.z);
      ray.init(scene.eye, dir);
      rayObject = ray.trace(scene.objects);
      dest[x] = RGBfCol.toRGB(rayObject != null ? rayObject.Shade(ray, scene, t1, t2, t3) : scene.background);
    }
  }
}
package raytracer;

/**
 * An object must implement a Renderable interface in order to be ray traced. It
 * should be straightforward to add new objects by implementing Renderable.
 */
public abstract class Renderable {
  /**
   * Checks whether the given ray ever intersects this object.
   * 
   * @param Ray
   *          to check against the object
   * @return float the closest t value at intersection, or -1 if there is no
   *         intersection.
   */
  public abstract float intersect(Ray r);

  /**
   * Shade a surface, given the world description and the ray.
   * 
   * @param Ray
   *          the ray to trace
   * @param Scene
   *          lights, objects, background
   * @param three
   *          temporary vectors (are changed in method!)
   * @return RGBfCol shading color
   */
  public abstract long Shade(Ray r, Scene scene, Vector3f t1, Vector3f t2,
      Vector3f t3);

  /**
   * Calculate an onPoint to be a small step away from the surface
   */
  public float onPoint(float t) {
    return t - (t / 10000.0f);
  }
}
package raytracer;

/**
 * RGBfCol
 * 
 * @author Stefan Frenz
 * @version 0.000.03.SF
 */

public class RGBfCol {
  private final static int RED = 2, GREEN = 1, BLUE = 0; // order in long

  private final static int bpc = 21; // 21 bits per color (maximum resolution:
                                      // 21 bits, care sign!)

  private final static float f_mul = 2097152.0f; // 2**21

  private final static int i_max = 2097151; // 2**21-1 == 0x1FFFFF (used as
                                            // mask)

  public static long create(float rI, float gI, float bI) {
    int r, g, b;

    r = (int) (rI * f_mul);
    g = (int) (gI * f_mul);
    b = (int) (bI * f_mul);
    if (r < 0)
      r = 0;
    else if (r > i_max)
      r = i_max;
    if (g < 0)
      g = 0;
    else if (g > i_max)
      g = i_max;
    if (b < 0)
      b = 0;
    else if (b > i_max)
      b = i_max;

    return (((long) r) << (bpc * 2)) | (((long) g) << bpc) | ((long) b);
  }

  private static int getIntCol(long col, int index) {
    int shift, val;

    shift = index * bpc;
    val = ((int) (col >>> shift)) & i_max;
    return val;
  }

  public static float fGetRed(long col) {
    float v;
    v = (float) getIntCol(col, RED);
    return v / f_mul;
  }

  public static float fGetGreen(long col) {
    float v;
    v = (float) getIntCol(col, GREEN);
    return v / f_mul;
  }

  public static float fGetBlue(long col) {
    float v;
    v = (float) getIntCol(col, BLUE);
    return v / f_mul;
  }

  public static int toRGB(long col) {
    int rv, gv, bv;

    rv = getIntCol(col, RED) >> (bpc - 8);
    gv = getIntCol(col, GREEN) >> (bpc - 8);
    bv = getIntCol(col, BLUE) >> (bpc - 8);

    return (rv << 16) | (gv << 8) | bv;
  }
}
package raytracer;

/**
 * The scene class encapsulates camera and world data.
 */

public class Scene {
  /** eye position */
  public Vector3f eye;

  /** direction of view */
  public Vector3f lookat;

  /** up vector from eye position */
  public Vector3f up;

  /** field of view */
  public float fov;

  /** list of objects in the world */
  public ObjList objects;

  /** list of lights in the world */
  public ObjList lights;

  /** the RGBfCol coded color of the scene background */
  public long background;

  /**
   * Scene constructor reads all camera and world details from a file.
   */
  public Scene() {
    // Initialize various lists
    objects = new ObjList();
    lights = new ObjList();

    setUpEye();
    setUpEnvironment();
    createWalls();
    createString();
    createThings();
    createLights();
  }

  private void setUpEye() {
    eye = new Vector3f(0.0f, -8.0f, 3.0f);
    up = new Vector3f(0.0f, 0.0f, 1.0f);
    lookat = new Vector3f(0.0f, -2.0f, 0.0f);
    fov = 75.0f;
  }

  private void setUpEnvironment() {
    Surface s;

    background = RGBfCol.create(0.078f, 0.361f, 0.753f);
    // background = RGBfCol.create(0.0f, 0.0f, 0.0f);
    s = new Surface(0.1f, 0.6f, 0.3f, 0.15f, 1.0f, 0.0f, 1.0f, 0.0f, 0.0f, 1.0f);
    objects.addElement(new Sphere(s, new Vector3f(0.0f, 0.0f, -2000.0f),
        2000.0f));
  }

  private void createWalls() {
    Surface s;
    Vector3f b1, b2, b3, b4, t1, t2, t3, t4;
    Vector3f mb1, mb2, mt1, mt2;

    b1 = new Vector3f(-2.0f, -2.0f, 0.0f);
    t1 = new Vector3f(-2.0f, -2.0f, 2.0f);
    b2 = new Vector3f(-2.0f, 2.0f, 0.0f);
    t2 = new Vector3f(-2.0f, 2.0f, 2.0f);
    b3 = new Vector3f(2.0f, 2.0f, 0.0f);
    t3 = new Vector3f(2.0f, 2.0f, 2.0f);
    b4 = new Vector3f(2.0f, -2.0f, 0.0f);
    t4 = new Vector3f(2.0f, -2.0f, 2.0f);
    mb1 = new Vector3f(-8.0f, 4.0f, 0.0f);
    mt1 = new Vector3f(-8.5f, 3.75f, 3.0f);
    mb2 = new Vector3f(-4.0f, 6.0f, 0.0f);
    mt2 = new Vector3f(-4.5f, 5.75f, 3.0f);

    s = new Surface(0.9f, 0.9f, 0.9f, 0.2f, 1.0f, 0.0f, 0.8f, 0.0f, 0.0f, 1.0f);
    objects.addElement(new Triangle(s, b1, b2, t1));
    objects.addElement(new Triangle(s, t1, t2, b2));
    objects.addElement(new Triangle(s, b3, b4, t3));
    objects.addElement(new Triangle(s, t3, t4, b4));

    s = new Surface(0.5f, 0.45f, 0.35f, 0.07f, 1.0f, 0.8f, 3.0f, 0.5f, 0.0f,
        1.0f);
    objects.addElement(new Triangle(s, b2, b3, t3));
    objects.addElement(new Triangle(s, t2, t3, b2));

    s = new Surface(0.0f, 0.0f, 0.0f, 0.07f, 1.0f, 0.8f, 3.0f, 0.5f, 0.0f, 1.0f);
    objects.addElement(new Triangle(s, mb1, mb2, mt1));
    objects.addElement(new Triangle(s, mt1, mt2, mb2));
  }

  private void createString() {
    Surface s;

    s = new Surface(0.5f, 0.45f, 0.35f, 0.07f, 1.0f, 0.8f, 3.0f, 0.5f, 0.0f,
        1.0f);
    objects.addElement(new Sphere(s, new Vector3f(-2.0f, 2.0f, 2.0f), 0.5f));
    objects.addElement(new Sphere(s, new Vector3f(-1.0f, 2.0f, 2.0f), 0.5f));
    objects.addElement(new Sphere(s, new Vector3f(0.0f, 2.0f, 2.0f), 0.5f));
    objects.addElement(new Sphere(s, new Vector3f(1.0f, 2.0f, 2.0f), 0.5f));
    objects.addElement(new Sphere(s, new Vector3f(2.0f, 2.0f, 2.0f), 0.5f));
  }

  private void createGroup(Surface cS, float x, float y, boolean upper) {
    objects.addElement(new Sphere(cS, new Vector3f(x + 0.0f, y + 0.0f, 0.5f),
        0.5f));
    objects.addElement(new Sphere(cS, new Vector3f(x + 0.643951f,
        y + 0.172546f, 0.5f), 0.166667f));
    objects.addElement(new Sphere(cS, new Vector3f(x + 0.172546f,
        y + 0.643951f, 0.5f), 0.166667f));
    objects.addElement(new Sphere(cS, new Vector3f(x - 0.471405f,
        y + 0.471405f, 0.5f), 0.166667f));
    objects.addElement(new Sphere(cS, new Vector3f(x - 0.643951f,
        y - 0.172546f, 0.5f), 0.166667f));
    objects.addElement(new Sphere(cS, new Vector3f(x - 0.172546f,
        y - 0.643951f, 0.5f), 0.166667f));
    objects.addElement(new Sphere(cS, new Vector3f(x + 0.471405f,
        y - 0.471405f, 0.5f), 0.166667f));
    if (upper) {
      objects.addElement(new Sphere(cS, new Vector3f(x + 0.272166f,
          y + 0.272166f, 1.044331f), 0.166667f));
      objects.addElement(new Sphere(cS, new Vector3f(x - 0.371785f,
          y + 0.0996195f, 1.044331f), 0.166667f));
      objects.addElement(new Sphere(cS, new Vector3f(x - 0.0996195f,
          y - 0.371785f, 1.044331f), 0.166667f));
    }
  }

  private void createThings() {
    Surface s;

    s = new Surface(0.0f, 0.0f, 1.0f, 0.07f, 1.0f, 0.8f, 3.0f, 0.5f, 0.0f, 1.0f);
    objects.addElement(new Sphere(s, new Vector3f(-1.5f, -1.5f, 0.5f), 0.5f));

    s = new Surface(1.0f, 0.0f, 0.0f, 0.07f, 1.0f, 0.8f, 3.0f, 0.5f, 0.0f, 1.0f);
    objects.addElement(new Sphere(s, new Vector3f(-1.0f, 0.0f, 0.5f), 0.5f));

    s = new Surface(0.5f, 0.45f, 0.35f, 0.07f, 1.0f, 0.8f, 3.0f, 0.5f, 0.0f,
        1.0f);
    objects.addElement(new Sphere(s, new Vector3f(0.0f, 1.0f, 0.5f), 0.5f));

    s = new Surface(1.0f, 1.0f, 0.0f, 0.07f, 1.0f, 0.8f, 3.0f, 0.5f, 0.0f, 1.0f);
    objects.addElement(new Sphere(s, new Vector3f(1.0f, 0.0f, 0.5f), 0.5f));

    s = new Surface(0.0f, 1.0f, 1.0f, 0.07f, 1.0f, 0.8f, 3.0f, 0.5f, 0.0f, 1.0f);
    objects.addElement(new Sphere(s, new Vector3f(1.5f, -1.5f, 0.5f), 0.5f));

    s = new Surface(1.0f, 1.0f, 1.0f, 0.07f, 1.0f, 0.8f, 5.0f, 0.6f, 0.0f, 1.0f);
    objects.addElement(new Cylinder(s, new Vector3f(3.0f, -1.8f, 1.2f), 0.35f,
        new Vector3f(-3.0f, 1.6f, 0.3f), false));

    s = new Surface(0.5f, 0.45f, 0.35f, 0.07f, 1.0f, 0.8f, 3.0f, 0.5f, 0.0f,
        1.0f);
    createGroup(s, 4.5f, 2.0f, false);
    createGroup(s, -4.5f, 2.0f, false);
    createGroup(s, 6.0f, 0.5f, true);
    createGroup(s, -6.0f, 0.5f, true);
    createGroup(s, 4.5f, -1.0f, false);
    createGroup(s, -4.5f, -1.0f, false);
    createGroup(s, 3.0f, -2.5f, true);
    createGroup(s, -3.0f, -2.5f, true);
    createGroup(s, 1.5f, -4.0f, false);
    createGroup(s, -1.5f, -4.0f, false);
    createGroup(s, 0.0f, -5.5f, true);
  }

  private void createLights() {
    lights.addElement(new Light(Light.AMBIENT, null, 0.5f, 0.5f, 0.5f));
    lights.addElement(new Light(Light.POINT, new Vector3f(10.0f, -3.0f, 10.0f),
        0.8f, 0.8f, 0.8f));
    lights.addElement(new Light(Light.POINT, new Vector3f(0.0f, 0.0f, 2.0f),
        0.3f, 0.3f, 0.3f));
  }
}
package raytracer;

/**
 * Sphere is an example "Renderable" object. Spheres are about as easy as it
 * gets when it comes to intersection.
 */
public class Sphere extends Renderable {

  /** The Sphere's surface description */
  public Surface surface;

  /** The center point of the sphere */
  public Vector3f center;

  /** The radius of the sphere */
  public float radius;

  /** Precomputed square of the radius */
  public float radSqr;

  /**
   * Sole constructor for a sphere
   * 
   * @param Surface
   *          the sphere's surface
   * @param Vector3f
   *          the center of the sphere
   * @param float
   *          the radius of the sphere
   */
  public Sphere(Surface s, Vector3f c, float r) {
    surface = s;
    center = c;
    radius = r;
    radSqr = r * r;
  }

  /**
   * Check whether this object intersects with the given ray.
   * 
   * @param Ray
   *          ray to intersect
   * @return float -1 if no intersection, otherwise returns value of parametric
   *         value t at the intersection point
   */
  public float intersect(Ray ray) {
    float dx = center.x - ray.origin.x;
    float dy = center.y - ray.origin.y;
    float dz = center.z - ray.origin.z;
    float v = ray.direction.dot(dx, dy, dz);
    float t;

    // Do the following quick check to see if there is even a chance
    // that an intersection here might be closer than a previous one
    if (v - radius > ray.t)
      return -1.0f;

    // Test if the ray actually intersects the sphere
    t = radSqr + v * v - dx * dx - dy * dy - dz * dz;
    if (t < 0.0f)
      return -1.0f;

    // Test if the intersection is in the positive
    // ray direction and it is the closest so far
    t = v - ((float) Math.sqrt((double) t));
    // t = v - (FPU.sqrt(t)); //buggy
    if ((t > ray.t) || (t < 0.0f))
      return -1.0f;

    return t;
  }

  /**
   * The shade function takes the description of the world and feeds data to a
   * surface shader. This function determines the point of intersection, surface
   * normal, and ray origin and passes them to the surface shader.
   * 
   * @param Ray
   *          the intersecting ray
   * @param Scene
   *          lights, objects, background
   * @return RGBfCol the shaded color
   */
  public long Shade(Ray ray, Scene scene, Vector3f t1, Vector3f t2, Vector3f t3) {
    // An object shader doesn't really do too much other than
    // supply a few critical pieces of geometric information
    // for a surface shader. It must must compute
    // 1. the point of intersection (p)
    // 2. a unit-length surface normal (n)
    // 3. a unit-length vector towards the ray's origin (v)
    float px = ray.origin.x + ray.t * ray.direction.x;
    float py = ray.origin.y + ray.t * ray.direction.y;
    float pz = ray.origin.z + ray.t * ray.direction.z;
    long col;

    t1.init(px, py, pz);
    t2.init(px - center.x, py - center.y, pz - center.z);
    t2.normalize();
    t3.init(-ray.direction.x, -ray.direction.y, -ray.direction.z);

    // The illumination model is applied
    // by the surface's Shade() method
    col = surface.Shade(t1, t2, t3, scene, ray);

    return col;
  }
}
package raytracer;

/**
 * Surface qualities associated with a Renderable object. All objects have a
 * single homogeneous surface.
 */
public class Surface {
  /** surface's intrinsic color */
  public float ir, ig, ib;

  /** phong model: ambient coefficent of surface */
  public float ka;

  /** phong model: diffuse coefficent of surface */
  public float kd;

  /** phong model: specular coefficient of surface */
  public float ks;

  /** phong model: shininess of surface */
  public float ns;

  /**
   * fraction of the transmitted ray's intensity that is reflected to the eye
   */
  public float kt;

  /**
   * Reflective constant of surface. Shininess only affects reflection of
   * lights, whereas this value affects reflection of the sight ray.
   */
  public float kr;

  /**
   * ratio of the surrounding material's index of refraction to the object's
   * index of refraction. Nt is Ni over Nr. Ni is the index of refraction for
   * air, that is, 1. Nr is the index of refraction for the object. Water, for
   * example, has an index of 1.33. Nt would be 1/1.33.
   */
  public float nt;

  /** handy small number */
  private static final float TINY = 0.00001f;

  /**
   * Sole constructor, which builds the Surface instance by being given all
   * values. Notable parameter is the reflection value, which is divided by 255
   * automatically.
   * 
   * @param float
   *          rval red
   * @param float
   *          gval green
   * @param float
   *          bval blue
   * @param float
   *          a ambient coefficient
   * @param float
   *          d diffuse coefficient
   * @param float
   *          s specular coefficient
   * @param float
   *          n shininess
   * @param float
   *          r reflectivity
   * @param float
   *          t transparency
   * @param float
   *          nt refractive index
   */

  public Surface(float rval, float gval, float bval, float a, float d, float s,
      float n, float r, float t, float index) {
    ir = rval;
    ig = gval;
    ib = bval;
    ka = a;
    kd = d;
    ks = s;
    ns = n;
    kr = r;
    kt = t;
    nt = index;
  }

  /**
   * This is the true shading function where work gets done. This function
   * determines the color value of a ray hitting this surface by calculating the
   * phong illumination model and spawning the appropriate secondary rays.
   * 
   * @return RGBfCol the color of the surface at the intersection pt
   * @param Vector3f
   *          p the point of intersection
   * @param Vector3f
   *          n the unit-length surface normal
   * @param Vector3f
   *          v the unit-length vector towards the ray origin
   * @param Scene
   *          lights, objects, color
   */
  public long Shade(Vector3f t1, Vector3f t2, Vector3f t3, Scene scene,
      Ray tempRay) {
    // start with shadow vectors to each light source
    ObjList lightSources;

    float r = 0.0f;
    float g = 0.0f;
    float b = 0.0f;

    // block-locals
    float lambert;
    float diffuse;
    float spec;
    float t;
    long rcolor;
    Light light;
    Renderable reflectedObject;

    // for optimization: copy values from p, n, v and use them as tempvars
    float px, py, pz, nx, ny, nz, vx, vy, vz;

    px = t1.x;
    py = t1.y;
    pz = t1.z;
    nx = t2.x;
    ny = t2.y;
    nz = t2.z;
    vx = t3.x;
    vy = t3.y;
    vz = t3.z;

    lightSources = scene.lights;
    while (lightSources != null) {
      light = (Light) lightSources.elem;
      lightSources = lightSources.next;
      if (light.lightType == Light.AMBIENT) {
        r += ka * ir * light.ir;
        g += ka * ig * light.ig;
        b += ka * ib * light.ib;
      }
      else {
        if (light.lightType == Light.POINT) {
          t1.init(light.lvec.x - px, light.lvec.y - py, light.lvec.z - pz);
          t1.normalize();
        }
        else
          t1.init(-light.lvec.x, -light.lvec.y, -light.lvec.z);

        // Check if the surface point is in shadow
        t2.init(px + TINY * t1.x, py + TINY * t1.y, pz + TINY * t1.z);
        tempRay.init(t2, t1);

        if (tempRay.cast(scene.objects) == false) { // direct ray to light
          lambert = t1.dot(nx, ny, nz);
          if (lambert > 0.0f) {
            if (kd > 0.0f) {
              diffuse = kd * lambert;
              r += diffuse * ir * light.ir;
              g += diffuse * ig * light.ig;
              b += diffuse * ib * light.ib;
            }
            if (ks > 0.0f) {
              lambert *= 2.0f;
              t3.init(lambert * nx - t1.x, lambert * ny - t1.y, lambert * nz
                  - t1.z);
              spec = t3.dot(vx, vy, vz);
              if (spec > 0.0f) {
                spec = ks * (float) Math.pow((double) spec, (double) ns);
                r += spec * light.ir;
                g += spec * light.ig;
                b += spec * light.ib;
              }
            }
          }
        }
      }
    }

    // Compute illumination due to reflection
    if (kr > 0.0f) {
      t1.init(vx, vy, vz);
      t = t1.dot(nx, ny, nz);
      if (t > 0.0f) {
        t *= 2.0f;
        t1.init(t * nx - vx, t * ny - vy, t * nz - vz);
        t2.init(px + TINY * t1.x, py + TINY * t1.y, pz + TINY * t1.z);
        tempRay.init(t2, t1);
        reflectedObject = tempRay.trace(scene.objects);
        if (reflectedObject != null) {
          t1.init(px, py, pz);
          t2.init(nx, ny, nz);
          t3.init(vx, vy, vz);
          rcolor = reflectedObject.Shade(tempRay, scene, t1, t2, t3);
          r += kr * RGBfCol.fGetRed(rcolor);
          g += kr * RGBfCol.fGetGreen(rcolor);
          b += kr * RGBfCol.fGetBlue(rcolor);
        }
        else {
          r += kr * RGBfCol.fGetRed(scene.background);
          g += kr * RGBfCol.fGetGreen(scene.background);
          b += kr * RGBfCol.fGetBlue(scene.background);
        }
      }
    }

    // Add code for refraction here!

    // Clamp RGB values
    /*
     * r = (r > 1f) ? 1f : r; g = (g > 1f) ? 1f : g; b = (b > 1f) ? 1f : b;
     */
    if (r > 1.0f)
      r = 1.0f;
    if (g > 1.0f)
      g = 1.0f;
    if (b > 1.0f)
      b = 1.0f;

    return RGBfCol.create(r, g, b);
  }
}
package raytracer;

/**
 * Triangle is an "Renderable" object.
 */
public class Triangle extends Renderable {
  /** epsilon */
  public final static float EPSILON = 0.0001f;

  /** The Triangle's surface description */
  public Surface surface;

  /** The Triangle's parameters */
  public Vector3f point1, point2, point3;

  public Triangle(Surface s, Vector3f v1, Vector3f v2, Vector3f v3) {
    surface = s;
    point1 = v1;
    point2 = v2;
    point3 = v3;
  }

  /**
   * Check whether this object intersects with the given ray.
   * 
   * @param Ray
   *          ray to intersect
   * @return float -1 if no intersection, otherwise returns value of parametric
   *         value t at the intersection point
   */
  public float intersect(Ray ray) {
    float t;
    float det, u, v;
    // following is to avoid "new" for new Vector3f
    float edge1x, edge1y, edge1z, edge2x, edge2y, edge2z;
    float px, py, pz, qx, qy, qz, tx, ty, tz;

    // edge1 = point2-point1
    edge1x = point2.x - point1.x;
    edge1y = point2.y - point1.y;
    edge1z = point2.z - point1.z;
    // edge2 = point3-point1
    edge2x = point3.x - point1.x;
    edge2y = point3.y - point1.y;
    edge2z = point3.z - point1.z;

    // pvec = cross(ray.direction, edge2)
    px = ray.direction.y * edge2z - ray.direction.z * edge2y;
    py = ray.direction.z * edge2x - ray.direction.x * edge2z;
    pz = ray.direction.x * edge2y - ray.direction.y * edge2x;

    // det = edge1.dot(pvec)
    det = edge1x * px + edge1y * py + edge1z * pz;
    if ((float) Math.abs((double) det) < EPSILON)
      return -1.0f;

    // calculate distance from point0 to ray origin
    // tvec = ray.origin-point1
    tx = ray.origin.x - point1.x;
    ty = ray.origin.y - point1.y;
    tz = ray.origin.z - point1.z;

    // calculate u parameter and test bounds
    // u = t.dot(p)
    u = (tx * px + ty * py + tz * pz) / det;
    if (u < 0.0f || u > 1.0f)
      return -1.0f;

    // prepare to test v parameter
    // qvec = t.cross(edge1)
    qx = ty * edge1z - tz * edge1y;
    qy = tz * edge1x - tx * edge1z;
    qz = tx * edge1y - ty * edge1x;

    // calculate v parameter and test bounds
    // v = ray.direction.dot(qvec)
    v = ray.direction.dot(qx, qy, qz) / det;
    if (v < 0.0f || u + v > 1.0f)
      return -1.0f;

    // calculate t parameter
    // t = edge2.dot(qvec)/det;
    t = (edge2x * qx + edge2y * qy + edge2z * qz) / det;

    if (ray.t < t)
      return -1.0f;
    return t;
  }

  /**
   * The shade function takes the description of the world and feeds data to a
   * surface shader. This function determines the point of intersection, surface
   * normal, and ray origin and passes them to the surface shader.
   * 
   * @param Ray
   *          the intersecting ray
   * @param Scene
   *          lights, objects, background
   * @return RGBfCol the shaded color
   */
  public long Shade(Ray ray, Scene scene, Vector3f t1, Vector3f t2, Vector3f t3) {
    // An object shader doesn't really do too much other than
    // supply a few critical pieces of geometric information
    // for a surface shader. It must must compute
    // 1. the point of intersection (p)
    // 2. a unit-length surface normal (n)
    // 3. a unit-length vector towards the ray's origin (v)
    float t = onPoint(ray.t);
    float px = ray.origin.x + t * ray.direction.x;
    float py = ray.origin.y + t * ray.direction.y;
    float pz = ray.origin.z + t * ray.direction.z;
    float edge1x, edge1y, edge1z, edge2x, edge2y, edge2z;
    float nx, ny, nz, l1, l2;
    long col;

    // edge1 = point2-point1
    edge1x = point2.x - point1.x;

    edge1y = point2.y - point1.y;
    edge1z = point2.z - point1.z;
    // edge2 = point3-point1
    edge2x = point3.x - point1.x;
    edge2y = point3.y - point1.y;
    edge2z = point3.z - point1.z;

    nx = edge1y * edge2z - edge1z * edge2y;
    ny = edge1z * edge2x - edge1x * edge2z;
    nz = edge1x * edge2y - edge1y * edge2x;
    l1 = (ray.origin.x - px - nx) * (ray.origin.x - px - nx)
        + (ray.origin.y - py - ny) * (ray.origin.y - py - ny)
        + (ray.origin.z - pz - nz) * (ray.origin.z - pz - nz);
    l2 = (ray.origin.x - px + nx) * (ray.origin.x - px + nx)
        + (ray.origin.y - py + ny) * (ray.origin.y - py + ny)
        + (ray.origin.z - pz + nz) * (ray.origin.z - pz + nz);
    if (l2 < l1) {

      nx = -nx;
      ny = -ny;
      nz = -nz;
    }

    t1.init(px, py, pz);
    t2.init(nx, ny, nz);
    t2.normalize();
    t3.init(-ray.direction.x, -ray.direction.y, -ray.direction.z);

    col = surface.Shade(t1, t2, t3, scene, ray);

    return col;
  }
}
package raytracer;

public class Vector3f {
  public float x, y, z;

  public Vector3f() {
    init(0.0f, 0.0f, 0.0f);
  }

  public Vector3f(Vector3f v1) {
    init(v1.x, v1.y, v1.z);
  }

  public Vector3f(float xI, float yI, float zI) {
    init(xI, yI, zI);
  }

  public void init(float xI, float yI, float zI) {
    x = xI;
    y = yI;
    z = zI;
  }

  public void normalize() {
    normalize(this);
  }

  public void normalize(Vector3f v1) {
    float len = (float) Math.sqrt((double) v1.lenSq());

    x = v1.x / len;
    y = v1.y / len;
    z = v1.z / len;
  }

  public float lenSq() {
    return x * x + y * y + z * z;
  }

  public void scale(float factor) {
    x *= factor;
    y *= factor;
    z *= factor;
  }

  public void add(Vector3f v1) {
    x += v1.x;
    y += v1.y;
    z += v1.z;
  }

  public void add(float vx, float vy, float vz) {
    x += vx;
    y += vy;
    z += vz;
  }

  public void addScaled(Vector3f v1, float factor) {
    x += factor * v1.x;
    y += factor * v1.y;
    z += factor * v1.z;
  }

  public void sub(Vector3f v1) {
    x -= v1.x;
    y -= v1.y;
    z -= v1.z;
  }

  public float dot(Vector3f v1) {
    return dot(v1.x, v1.y, v1.z);
  }

  public float dot(float vx, float vy, float vz) {
    return x * vx + y * vy + z * vz;
  }

  public void cross(Vector3f v1, Vector3f v2) {
    x = v1.y * v2.z - v1.z * v2.y;
    y = v1.z * v2.x - v1.x * v2.z;
    z = v1.x * v2.y - v1.y * v2.x;
  }

  // functions to avoid Vector3f-objects

  public static float lenSq(float x, float y, float z) {
    return x * x + y * y + z * z;
  }
}
